﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;
using System.IO.Ports;

public class SerialLaser : MonoBehaviour
{
    // Gloal vairables selectable from Unity
    public string Com_Port = "COM6";
    public int Baud_Rate = 115200;

    // Global parameter values
    public int devVal = 10;
    public int setPoint = 175;
    public int maxAcc = 25;
    public int accZone = 100;

    SerialPort sp = new SerialPort();


    // Start is called before the first frame update
    void Start()
    {

        try
        {
            sp.BaudRate = Baud_Rate;
            sp.PortName = Com_Port;
            sp.ReadTimeout = 1;

            sp.Open();
        }
        catch
        {

            Debug.Log("Serial port error!");
        }
    }

    // Update is called once per frame
    void Update()
    {
        try
        {
            string readInput = sp.ReadLine();
            readInput = readInput.Replace("\r", "");

            int readVal = Convert.ToInt32(readInput);

            calculateMovement(readVal);


            // Keyboard outputs to hardware
            if (sp.IsOpen)
            {
                // reads both the numbers above the keys or numberpad
                if (Input.GetKeyUp("1") || Input.GetKeyUp("[1]"))
                {
                    sp.Write("1");
                    Debug.Log(1);
                }
                else if (Input.GetKeyUp("0") || Input.GetKeyUp("[0]"))
                {
                    sp.Write("0");
                    Debug.Log(0);
                }
            }
        }
        catch
        {

        }

    }

    void calculateMovement(int input)
    {
        if (input > 0)
        {
            // get the raw acc value in respect to the setpoint
            float accVal = input - setPoint;

            decimal accelerationTmp = 0;

            decimal multiplyer = 0.00M;
            multiplyer = Decimal.Divide(maxAcc, accZone);

            //calculate the acceleration
            if (accVal > 0 && accVal > devVal) // if positive and outside noise level
            {
                // offset the acc val to start at 0
                accVal = accVal - devVal;

                // map the value to the range
                accelerationTmp = (decimal)accVal * multiplyer;

                // constrain output to limits
                if (accelerationTmp > maxAcc) accelerationTmp = maxAcc;

                //moveLeft((int)accelerationTmp);
                moveBackward((int)accelerationTmp);
            }
            else if (accVal < 0 && accVal < -devVal) // if negative and outside noise level
            {
                // offset the acc val to start at 0
                accVal = accVal + devVal;

                // map the value to the range
                accelerationTmp = -(decimal)accVal * multiplyer;

                // constrain output to limits
                if (accelerationTmp > maxAcc) accelerationTmp = maxAcc;

                //moveRight((int)accelerationTmp);
                moveForward((int)accelerationTmp);
            }
        }
        else
        {
            // Set outputs to 0
        }
    }

    void moveLeft(int moveVal)
    {
        transform.Translate(Vector3.left * moveVal, Space.World);
    }
    void moveRight(int moveVal)
    {
        transform.Translate(Vector3.right * moveVal, Space.World);
    }
    void moveForward(int moveVal)
    {
        transform.Translate(Vector3.forward * moveVal, Space.World);
    }
    void moveBackward(int moveVal)
    {
        transform.Translate(Vector3.back * moveVal, Space.World);
    }
}
